@@ -1,4 +1,4 @@
-;;; wl-e21.el -- Wanderlust modules for Emacs 21.
+;;; wl-e21.el --- Wanderlust modules for Emacs 21.
 
 ;; Copyright (C) 2000,2001 Katsumi Yamaoka <yamaoka@jpl.org>
 ;; Copyright (C) 2000,2001 Yuuichi Teranishi <teranisi@gohome.org>
@@ -33,7 +33,7 @@
 ;;(let (image icon from to overlay)
 ;;  ;; The function `find-image' will look for an image first on `load-path'
 ;;  ;; and then in `data-directory'.
-;;  (let ((load-path (cons wl-icon-dir load-path)))
+;;  (let ((load-path (cons wl-icon-directory load-path)))
 ;;    (setq image (find-image (list (list :type 'xpm :file wl-nntp-folder-icon
 ;;					:ascent 'center)))))
 ;;  ;; `propertize' is a convenient function in such a case.
@@ -62,6 +62,7 @@
 ;;; Code:
 ;;
 
+(require 'elmo)
 (eval-when-compile
   (require 'wl-folder)
   (require 'wl-summary)
@@ -71,15 +72,6 @@
   (defvar-maybe wl-folder-mode-map (make-sparse-keymap))
   (defvar-maybe wl-draft-mode-map (make-sparse-keymap)))
 
-;; For Emacs 21.0.104 or earlier
-(defun-maybe make-mode-line-mouse-map (mouse function) "\
-Return a keymap with single entry for mouse key MOUSE on the mode line.
-MOUSE is defined to run function FUNCTION with no args in the buffer
-corresponding to the mode line clicked."
-  (let ((map (make-sparse-keymap)))
-    (define-key map (vector 'mode-line mouse) function)
-    map))
-
 (add-hook 'wl-folder-mode-hook 'wl-setup-folder)
 (add-hook 'wl-folder-mode-hook 'wl-folder-init-icons)
 
@@ -88,6 +80,8 @@ corresponding to the mode line clicked."
 
 (add-hook 'wl-summary-mode-hook 'wl-setup-summary)
 
+(add-hook 'wl-message-display-internal-hook 'wl-setup-message)
+
 (defvar wl-use-toolbar (image-type-available-p 'xpm))
 (defvar wl-plugged-image nil)
 (defvar wl-unplugged-image nil)
@@ -176,18 +170,18 @@ corresponding to the mode line clicked."
 
 (eval-when-compile
   (defmacro wl-e21-display-image-p ()
-    '(and (display-graphic-p)
+    '(and (display-images-p)
 	  (image-type-available-p 'xpm))))
 
 (defun wl-e21-setup-toolbar (bar)
   (when (and wl-use-toolbar
 	     (wl-e21-display-image-p))
-    (let ((load-path (cons wl-icon-dir load-path))
+    (let ((load-path (cons wl-icon-directory load-path))
 	  (props '(:type xpm :ascent center
 			 :color-symbols (("backgroundToolBarColor" . "None"))
 			 :file))
 	  (success t)
-	  icon up down disabled name success)
+	  icon up down disabled name)
       (while bar
 	(setq icon (aref (pop bar) 0))
 	(unless (boundp icon)
@@ -239,14 +233,14 @@ corresponding to the mode line clicked."
     (wl-e21-make-toolbar-buttons wl-summary-mode-map wl-summary-toolbar)))
 
 (eval-when-compile
-  (defsubst wl-e21-setup-message-toolbar (keymap)
-    (when (wl-e21-setup-toolbar wl-message-toolbar)
-      (wl-e21-make-toolbar-buttons keymap wl-message-toolbar)))
-
   (defsubst wl-e21-setup-draft-toolbar ()
     (when (wl-e21-setup-toolbar wl-draft-toolbar)
       (wl-e21-make-toolbar-buttons wl-draft-mode-map wl-draft-toolbar))))
 
+(defun wl-e21-setup-message-toolbar ()
+  (when (wl-e21-setup-toolbar wl-message-toolbar)
+    (wl-e21-make-toolbar-buttons (current-local-map) wl-message-toolbar)))
+
 (defvar wl-folder-toggle-icon-list
   '((wl-folder-opened-image       . wl-opened-group-folder-icon)
     (wl-folder-closed-image       . wl-closed-group-folder-icon)))
@@ -266,7 +260,7 @@ corresponding to the mode line clicked."
 	  (unless image
 	    (let ((name (symbol-value
 			 (cdr (assq icon wl-folder-toggle-icon-list))))
-		  (load-path (cons wl-icon-dir load-path)))
+		  (load-path (cons wl-icon-directory load-path)))
 	      (when (setq image (find-image `((:type xpm :file ,name
 						     :ascent center))))
 		(setq image (put icon 'image (propertize name
@@ -286,7 +280,7 @@ corresponding to the mode line clicked."
       (let ((inhibit-read-only t))
 	(if (and wl-highlight-folder-by-numbers
 		 numbers (nth 0 numbers) (nth 1 numbers)
-		 (re-search-forward "[0-9-]+/[0-9-]+/[0-9-]+"
+		 (re-search-forward "[-[:digit:]]+/[-[:digit:]]+/[-[:digit:]]+"
 				    (line-end-position) t))
 	    (let* ((unsync (nth 0 numbers))
 		   (unread (nth 1 numbers))
@@ -338,7 +332,7 @@ corresponding to the mode line clicked."
        (;; basic folder
 	(and (setq fld-name (wl-folder-get-folder-name-by-id
 			     (get-text-property (point) 'wl-folder-entity-id)))
-	     (looking-at "[\t ]+\\([^\t\n ]+\\)"))
+	     (looking-at "[[:blank:]]+\\([^[:blank:]\n]+\\)"))
 	(setq start (match-beginning 1)
 	      end (match-end 1))
 	(let (image)
@@ -366,7 +360,7 @@ corresponding to the mode line clicked."
 			    ((string= fld-name wl-queue-folder);; queue folder
 			     (get 'wl-folder-queue-image 'image))
 			    (;; and one of many other folders
-			     (setq type (elmo-folder-get-type fld-name))
+			     (setq type (elmo-folder-type fld-name))
 			     (get (intern (format "wl-folder-%s-image" type))
 				  'image)))))
 	      (overlay-put overlay 'before-string image)))
@@ -381,7 +375,7 @@ corresponding to the mode line clicked."
 	(when (display-color-p)
 	  (wl-e21-highlight-folder-by-numbers
 	   start end
-	   (if (looking-at (format "^[\t ]*\\(%s\\|%s\\)"
+	   (if (looking-at (format "^[[:blank:]]*\\(?:%s\\|%s\\)"
 				   wl-folder-unsubscribe-mark
 				   wl-folder-removed-mark))
 	       'wl-highlight-folder-killed-face
@@ -393,7 +387,7 @@ corresponding to the mode line clicked."
   (when (wl-e21-display-image-p)
     (save-excursion
       (beginning-of-line)
-      (when (looking-at "[\t ]*\\(\\[\\([^]]+\\)\\]\\)")
+      (when (looking-at "[[:blank:]]*\\(\\[\\([^]]+\\)\\]\\)")
 	(let* ((start (match-beginning 1))
 	       (end (match-end 1))
 	       (status (match-string-no-properties 2))
@@ -417,14 +411,12 @@ corresponding to the mode line clicked."
   (if (wl-e21-display-image-p)
       (let (type)
 	(cond ((string= folder wl-queue-folder)
-	       (concat (propertize " " 'display
-				   (get 'wl-folder-queue-image 'image))
+	       (concat (get 'wl-folder-queue-image 'image)
 		       string))
-	      ((setq type (elmo-folder-get-type folder))
-	       (concat (propertize " " 'display
-				   (get (intern (format "wl-folder-%s-image"
-							type))
-					'image))
+	      ((setq type (elmo-folder-type folder))
+	       (concat (get (intern (format "wl-folder-%s-image"
+					    type))
+			    'image)
 		       string))
 	      (t
 	       string)))
@@ -443,6 +435,8 @@ corresponding to the mode line clicked."
     (wl-folder-archive-image      . wl-archive-folder-icon)
     (wl-folder-pipe-image         . wl-pipe-folder-icon)
     (wl-folder-maildir-image      . wl-maildir-folder-icon)
+    (wl-folder-nmz-image          . wl-nmz-folder-icon)
+    (wl-folder-shimbun-image      . wl-shimbun-folder-icon)
     (wl-folder-trash-empty-image  . wl-empty-trash-folder-icon)
     (wl-folder-draft-image        . wl-draft-folder-icon)
     (wl-folder-queue-image        . wl-queue-folder-icon)
@@ -450,7 +444,7 @@ corresponding to the mode line clicked."
 
 (defun wl-folder-init-icons ()
   (when (wl-e21-display-image-p)
-    (let ((load-path (cons wl-icon-dir load-path))
+    (let ((load-path (cons wl-icon-directory load-path))
 	  (icons wl-folder-internal-icon-list)
 	  icon name image)
       (while (setq icon (pop icons))
@@ -468,7 +462,7 @@ corresponding to the mode line clicked."
     (if (wl-e21-display-image-p)
 	(progn
 	  (unless wl-plugged-image
-	    (let ((load-path (cons wl-icon-dir load-path)))
+	    (let ((load-path (cons wl-icon-directory load-path)))
 	      (setq wl-plugged-image (find-image
 				      `((:type xpm
 					       :file ,wl-plugged-icon
@@ -499,7 +493,7 @@ corresponding to the mode line clicked."
     (if (wl-e21-display-image-p)
 	(progn
 	  (unless wl-biff-mail-image
-	    (let ((load-path (cons wl-icon-dir load-path)))
+	    (let ((load-path (cons wl-icon-directory load-path)))
 	      (setq wl-biff-mail-image (find-image
 					`((:type xpm
 						 :file ,wl-biff-mail-icon
@@ -530,23 +524,24 @@ corresponding to the mode line clicked."
 
 (defalias 'wl-setup-summary 'wl-e21-setup-summary-toolbar)
 
-(defun wl-message-overload-functions ()
-  (let ((keymap (current-local-map)))
-    (when keymap
-      (wl-e21-setup-message-toolbar keymap)
-      (define-key keymap "l" 'wl-message-toggle-disp-summary)
-      (define-key keymap [mouse-2] 'wl-message-refer-article-or-url)
-      (define-key keymap [mouse-4] 'wl-message-wheel-down)
-      (define-key keymap [mouse-5] 'wl-message-wheel-up)
-      (define-key keymap [S-mouse-4] 'wl-message-wheel-down)
-      (define-key keymap [S-mouse-5] 'wl-message-wheel-up)
-      (set-keymap-parent wl-message-button-map keymap)
-      (define-key wl-message-button-map
-	[mouse-2] 'wl-message-button-dispatcher))))
+(defun wl-message-define-keymap ()
+  (let ((keymap (make-sparse-keymap)))
+    (define-key keymap "l" 'wl-message-toggle-disp-summary)
+    (define-key keymap [mouse-4] 'wl-message-wheel-down)
+    (define-key keymap [mouse-5] 'wl-message-wheel-up)
+    (define-key keymap [S-mouse-4] 'wl-message-wheel-down)
+    (define-key keymap [S-mouse-5] 'wl-message-wheel-up)
+    (set-keymap-parent wl-message-button-map keymap)
+    (define-key wl-message-button-map
+      [mouse-2] 'wl-message-button-dispatcher)
+    keymap))
+
+(defalias 'wl-setup-message 'wl-e21-setup-message-toolbar)
 
 (defun wl-message-wheel-up (event)
   (interactive "e")
-  (if (string-match wl-message-buf-name (buffer-name))
+  (if (string-match (regexp-quote wl-message-buffer-cache-name)
+		    (regexp-quote (buffer-name)))
       (wl-message-next-page)
     (let ((cur-buf (current-buffer))
 	  proceed)
@@ -561,7 +556,8 @@ corresponding to the mode line clicked."
 
 (defun wl-message-wheel-down (event)
   (interactive "e")
-  (if (string-match wl-message-buf-name (buffer-name))
+  (if (string-match (regexp-quote wl-message-buffer-cache-name)
+		    (regexp-quote (buffer-name)))
       (wl-message-prev-page)
     (let ((cur-buf (current-buffer))
 	  proceed)
@@ -601,7 +597,6 @@ Special commands:
 (defun wl-draft-key-setup ()
   (define-key wl-draft-mode-map "\C-c\C-y" 'wl-draft-yank-original)
   (define-key wl-draft-mode-map "\C-c\C-s" 'wl-draft-send)
-  (define-key wl-draft-mode-map "\C-c\C-a" 'wl-draft-insert-x-face-field)
   (define-key wl-draft-mode-map "\C-c\C-c" 'wl-draft-send-and-exit)
   (define-key wl-draft-mode-map "\C-c\C-z" 'wl-draft-save-and-exit)
   (define-key wl-draft-mode-map "\C-c\C-k" 'wl-draft-kill)
@@ -613,12 +608,13 @@ Special commands:
   (define-key wl-draft-mode-map "\C-c\C-e" 'wl-draft-config-exec)
   (define-key wl-draft-mode-map "\C-c\C-j" 'wl-template-select)
   (define-key wl-draft-mode-map "\C-c\C-p" 'wl-draft-preview-message)
+  (define-key wl-draft-mode-map "\C-c\C-a" 'wl-addrmgr)
   (define-key wl-draft-mode-map "\C-x\C-s" 'wl-draft-save)
   (define-key wl-draft-mode-map "\C-xk"    'wl-draft-mimic-kill-buffer))
 
 (defun wl-draft-overload-functions ()
   (wl-mode-line-buffer-identification)
-  (local-set-key "\C-c\C-s" 'wl-draft-send);; override
+;;  (local-set-key "\C-c\C-s" 'wl-draft-send);; override
   (wl-e21-setup-draft-toolbar)
   (wl-draft-overload-menubar))
 
